/*
 * IMPORTANT: DO NOT MODIFY libch.h ! USE INSTEAD libch.h.in !
 *
 *
 * This source code is part of my universal C library "libch".
 * Copyright (C) 1998,99  Christian Hammers <ch@westend.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Changelog:
 * 1998-12-30	added ch_chop()
 * 1998-10-04	now *sql.h is included here and setup by configure
 * 1998-09-27	added ch_sql_quiet_query
 * 1998-09-17	added ch_html_* functions
 * 1998-09-15	added ch_inet_validip()
 * 1998-06-20	added_ch_inet_* functions
 * 1998-03-28	added ch_time_* functions
 */

#ifndef __LIB_CH_H__
#define __LIB_CH_H__

#include <time.h>

typedef	char	string[1024];

extern int	ch_state_debugged;
extern int	ch_state_cgi_ctype;
extern int	ch_o_init_content_type;
extern int	ch_o_pid_logfile_names;

extern char    *ch_empty_string;	 	 /* ch_empty_string=="" */

/** ANSI C did not know them. (see string.h) 				*/
extern char *index(const char *s, int c);
extern char *rindex(const char *s, int c);

/************************************************************************/
/***			MISC						*/
/************************************************************************/

/*									*/
/* 	void ch_debug(char *s)						*/
/*		alias 							*/
/*	void ___(char *s)						*/
/* 									*/
/* 	Debug routine. Saves given strings in the /tmp/ch_debugfile.$$	*/
/*	that is newly created every start. Survives a CGI prg core dump */
/*									*/
void ch_debug(char *format, ...);
#define ____ ch_debug("LINE:%d",__LINE__);
#define ___ ch_debug

/** 	Removes ch_debugfile since program seems to work.  		*/ 
void ch_done(); 

/** 	Exits program with given string and logs reason.       	 	*/
void ch_panic(char *format, ...); 

/**	A normal malloc, but with retrun code check.              	*/
void *ch_malloc(size_t size); 

/**	A normal free, but sets the pointer to NULL.              	*/
void ch_free(void *pointer); 

/** 	Returns ENV string or "" but never NULL.         		*/
char *ch_getenv(const char *name); 


/************************************************************************/
/***			CGI						*/	
/************************************************************************/

/** 	0: ch_cgi_init does not print a content-type. 			*/
/** 	1: ch_cgi_init does set "text/html" content-type.		*/
/**	DEFAULT: 1							*/
/**	The ch_cgi_panic does set Content-Type anyways.			*/
void ch_option_init_content_type(int yesno);

/**	0: debug file is always called /tmp/ch_debugfile. 		*/
/** 	1: debug filenames are suffixed by the PID number.		*/
/**	DEFAULT: 1							*/
/**	Be aware that "0" is not multi-threading-safe !			*/
void ch_option_pid_logfile_names(int yesno);

/** 	Exits program with given string  and back button 		*/
void ch_cgi_panic(char *format, ...); 

/** 	Sets HTTP Content-Type header					*/
void ch_cgi_content_type(char *content_type); 

/** 	Initialise the cgi routienes (gets the content string)		*/
void ch_cgi_init(); 

/**     Includes a HTML file within the DOCUMENT_ROOT in CGI output     */
void ch_cgi_include(const char *fname);

/**     Dumps the ch_cgi_s string with all cgi values (DEBUG)           */
void ch_cgi_dump();

/**	Translates '+'  to ' ' and "%4D" to (char)0x4d			*/
char *ch_cgi_unescape(char *s);

/** 	Returns the value of the key <name>.   			  	*/
/**	If the name is non existent "" is returned.      		*/
void ch_cgi_get_value(char *name, char *buffer, int bufsize);

/** 	Returns a pointer to the value key <name>.		     	*/
/**	If the name is non existent "" is returned.      		*/
char *ch_cgi_value(char *name);

/** 	Returns 0 if value is NOT set else 1     			*/
int ch_cgi_isset_value(char *name); 

/**     Returns the CGI query string					*/
char *ch_cgi_get_string();

/**     Sets the CGI query string					*/
void ch_cgi_set_string(char *query_string);


/************************************************************************/
/***			HTML						*/	
/************************************************************************/

/* translates e.g. "" to "&ouml;" */
int ch_html_escape (char *out, const unsigned char *in, int maxout);
/* translates e.g. "&uuml;" to "" */
int ch_html_unescape (char *out, const unsigned char *in, int maxout);



/************************************************************************/
/***			TIME						*/	
/************************************************************************/

/* mostly selfexplaining. 
 *
 * * time_get() returns the # of seconds since Epoch (Unix Time)
 * 
 * * str2*() know about the following time representations:
 *  	970713 / 19970713 / 13.07.1997 / 13.07.1997 16:30
 *
*/ 

time_t ch_time_get();

int ch_time_tm2timet(struct tm *tm_in, time_t *timet_out);
int ch_time_tm2str(struct tm *tm_in, char *str_out);
int ch_time_timet2tm(time_t timet_in, struct tm *tm_out);
int ch_time_timet2str(time_t timet_in, char *str_out);
int ch_time_str2tm(char *str_in, struct tm *tm_out);
int ch_time_str2timet(char *str_in, time_t *timet_out);


/************************************************************************/
/***			SQL						*/	
/************************************************************************/

/* REMEMBER:
 * 
 * ch_sql_res_t *res
 * ch_sql_row_t row
 * 
 * // make query string  
 * ch_sql_query(query);
 * res=ch_sql_get_result(); 
 * row=ch_sql_fetch_row(res); 
 * 
 * print row[4];
 * 
 * ch_sql_free_result(res);
 *---------------
 * ch_sql_query terminates the program if an error occurs.
 * ch_sql_quiet_query just gives a return value and let you evaluate and
 * react upon the ch_sql_error() value.
 */

#ifdef CH_SQL_MYSQL
	#include "/usr/include/mysql/mysql.h"
	
	extern MYSQL ch_mysql_con;

	#define ch_sql_main_t		MYSQL
	#define ch_sql_res_t		MYSQL_RES
	#define ch_sql_row_t		MYSQL_ROW

	#define ch_sql_open		ch_mysql_open
	#define ch_sql_close		ch_mysql_close
	#define ch_sql_error		ch_mysql_error
	#define ch_sql_query		ch_mysql_query
	#define ch_sql_quiet_query	ch_mysql_query
	#define ch_sql_store_result 	ch_mysql_store_result
	#define ch_sql_num_rows		ch_mysql_num_rows
	#define	ch_sql_fetch_row	ch_mysql_fetch_row
	#define ch_sql_free_result	ch_mysql_free_result

	void 		ch_mysql_open(char *name);
	void 		ch_mysql_close();
	char 	       *ch_mysql_error();
	void 		ch_mysql_query(char *query);
	int 		ch_mysql_quiet_query(char *query);
	ch_sql_res_t   *ch_mysql_store_result();
	int 		ch_mysql_num_rows(ch_sql_res_t *result);
	ch_sql_row_t 	ch_mysql_fetch_row(ch_sql_res_t *result);
	void 		ch_mysql_free_result(ch_sql_res_t *result); 
#endif CH_SQL_MYSQL

#ifdef CH_SQL_MSQL
 	#include ""
	
	#define ch_sql_main_t	
	#define ch_sql_res_t		m_result
	#define ch_sql_row_t		m_row

	#define ch_sql_open		ch_msql_open
	#define ch_sql_close		ch_msql_close
	#define ch_sql_query		ch_msql_query
	#define ch_sql_quiet_query	ch_msql_query
	#define ch_sql_error		ch_msql_error
	#define ch_sql_store_result	ch_msql_store_result
	#define ch_sql_fetch_row	ch_msql_fetch_row
	#define ch_sql_num_rows		ch_msql_num_rows
	#define ch_sql_getid		ch_msql_getid
	#define ch_sql_free_result	ch_msql_free_result

	void 		ch_msql_open(char *name);
	void 		ch_msql_close();
	char 	       *ch_msql_error();
	int 		ch_msql_getid();
	void 		ch_msql_query(char *query);
	int 		ch_msql_quiet_query(char *query);
	ch_sql_res_t 	*ch_msql_store_result();
	int 		ch_msql_num_rows(ch_sql_res_t *result);
	ch_sql_row_t 	ch_msql_fetch_row(ch_sql_res_t *result);
	void 		ch_msql_free_result(ch_sql_res_t *result); 
#endif CH_SQL_MSQL

#ifdef CH_SQL_PGSQL
	#include "/usr/include/postgresql/libpq-fe.h"
	
	#define CH_PGSQL_MAX_FIELDS	100
	extern PGconn *ch_pgsql_con;
	
	typedef char **ch_pgsql_row_t;
	extern  char *ch_pgsql_row_buf[CH_PGSQL_MAX_FIELDS+1];

	#define ch_sql_main_t		PGcon
	#define ch_sql_res_t		PGresult
	#define ch_sql_row_t		ch_pgsql_row_t

	#define ch_sql_open		ch_pgsql_open
	#define ch_sql_close		ch_pgsql_close
	#define ch_sql_error		ch_pgsql_error
	#define ch_sql_query		ch_pgsql_query
	#define ch_sql_quiet_query	ch_pgsql_query
	#define ch_sql_store_result 	ch_pgsql_store_result
	#define ch_sql_num_rows		ch_pgsql_num_rows
	#define	ch_sql_fetch_row	ch_pgsql_fetch_row
	#define ch_sql_free_result	ch_pgsql_free_result

	void 		ch_pgsql_open(char *name);
	void 		ch_pgsql_close();
	char 	       *ch_pgsql_error();
	void 		ch_pgsql_query(char *query);
	int 		ch_pgsql_quiet_query(char *query);
	ch_sql_res_t   *ch_pgsql_store_result();
	int 		ch_pgsql_num_rows(ch_sql_res_t *result);
	ch_sql_row_t 	ch_pgsql_fetch_row(ch_sql_res_t *result);
	void 		ch_pgsql_free_result(ch_sql_res_t *result); 
#endif CH_SQL_PGSQL




/************************************************************************/
/***			INET						*/	
/************************************************************************/

/* ERROR CONSTANTS - the variable (int)ch_inet_error will be set with */
/* this values. NO libch routine will set this variable to 0 !!!      */
extern int ch_inet_error;
#define CH_INET_NOERROR 0
#define CH_INET_HOST_INVALIDCHAR 1	/* like german diaresis chars */
#define CH_INET_HOST_INVLEN 2		/* invalid length of hostname */
#define CH_INET_HOST_NUMDOTS 3		/* no dots in host.domain.tld */
#define CH_INET_HOST_LEADDASH 4		/* no leading dash allowed */
#define CH_INET_DOMAIN_NUMDOTS 5        /* no dots in domain.tld */
#define CH_INET_DOMAIN_INVALIDCHAR 6    /* like german diaresis chars */
#define CH_INET_DOMAIN_INVLEN 7         /* invalid length of domain name */
#define CH_INET_DOMAIN_LEADDASH 8       /* no leading dash allowed */
#define CH_INET_TLD_NUMDOTS 9		/* not the form a.b.c.domain.tld */
#define CH_INET_TLD_INVALIDCHAR 10      /* no dashes etc in TopLevelDomain */
#define CH_INET_TLD_INVLEN 11   	/* invalid length of tld */
#define CH_INET_TLD_NOCOM 12	        /* TopLevelDomain is no .com */
#define CH_INET_TLD_NODE 13	        /* TopLevelDomain is no .de */
#define CH_INET_BUFFEROVERFLOW 14       /* the string is too large */
#define CH_INET_IP_INVLEN 15		/* the IP is >16 chars or has too many dots */
#define CH_INET_IP_INVALIDCHAR 16	/* the IP has an invalid char */
#define CH_INET_IP_INVALIDNUMBER 17	/* the IP has an invalid number */
#define CH_INET_UNKNOWN 255

/* DEFINES */
#define CH_INET_HOST_MAXLEN 50		/* max length of hostname */


/* All ch_inet_* functions return 1 if they worked well or 0 if an error */
/* occured. In this case the variable ch_inet_error is set to non-zero ! */

/* extract HostName from string */
int ch_inet_gethost(char *src, char *dst, int maxlen);
/* extract DomainName from string */
int ch_inet_getdomain(char *src, char *dst, int maxlen);
/* extract TopLevelDomain from string */
int ch_inet_gettld(char *src, char *dst, int maxlen);

/* check if HostName is valid */
int ch_inet_validhost(char *host);
/* check if DomainName is valid */
int ch_inet_validdomain(char *domain);
/* check if TopLevelDomain is valid */
int ch_inet_validtld(char *tld);

/* check if a fully qualifying domain name is valid host.domain.tld */
int ch_inet_checkfqdn(char *s);
/* check if a domainname in the form of domain.tld is valid */
int ch_inet_checkdomain(char *s);
/* check if a domainname in the form of domain."de" is valid */
int ch_inet_checkdedomain(char *s);
/* check if a domainname in the form of domain."com" is valid */
int ch_inet_checkcomdomain(char *s); 

/* check if the IP number is valid */
int ch_inet_validip(char *ip);


/************************************************************************/
/***			REGEX						*/	
/************************************************************************/

/* returns 1 if the basic regular expression ex matches the string s */
int ch_regex_match(char *s, char *ex);


/************************************************************************/
/***			STRING						*/	
/************************************************************************/

/* usefull to cut trailing characters like '\n' or ' ' */
void ch_chop(char *s, char *bad_chars);


#endif __LIB_CH_H__

/* vim: syntax=c 
*/
